/*
 * Copyright (C) 2005 by egnite Software GmbH
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For additional information see http://jtagomat.sourceforge.net/
 */

#include "trace.h"

#define _BV(n)  (1 << (n))

#define ps_reg      ((volatile unsigned long *)0xFFFF4000)
#define AT91_PS_PCER    1
#define AT91_PS_PCDR    2

#define US0_ID          2       /* USART Channel 0 */
#define US1_ID          3       /* USART Channel 1 */

#define pio_reg  ((volatile unsigned long *)0xFFFF0000)
#define AT91_PIO_PDR    1
#define PIOTXD0         14         /* USART 0 transmit data signal */
#define PIORXD0         15         /* USART 0 receive data signal */

#define usart0_reg  ((volatile unsigned long *)0xFFFD0000)
#define AT91_US_CR              0   /* Control register */
#define AT91_US_RSTRX           2
#define AT91_US_RSTTX           3
#define AT91_US_RXEN            4
#define AT91_US_RXDIS           5
#define AT91_US_TXEN            6
#define AT91_US_TXDIS           7

#define AT91_US_MR              1  /* Mode register */
#define AT91_US_MR_LENGTH       6
#define AT91_US_MR_LENGTH_8     (3<<AT91_US_MR_LENGTH)
#define AT91_US_MR_PARITY       9
#define AT91_US_MR_PARITY_NONE  (4<<AT91_US_MR_PARITY)


#define AT91_US_IDR             3   /* Interrupt disable register */

#define AT91_US_CSR             5   /* Channel status register */
#define AT91_TXRDY              1   /* Transmit ready */

#define AT91_US_THR             7   /* Transmit holding register */

#define AT91_US_BRG             8   /* Baud rate generator */

#define AT91_US_RCR             13  /* Receive counter register */
#define AT91_US_TCR             15  /* Transmit counter register */


static char hexbuf[9];
static char hexdigit[] = "0123456789ABCDEF";

void TraceInit(void)
{
    /* Enable baud rate clocks. */
    ps_reg[AT91_PS_PCER] = _BV(US0_ID);
    ps_reg[AT91_PS_PCDR] = _BV(US1_ID);

    /* PIO disbale, enables preiphreal control. */
    pio_reg[AT91_PIO_PDR] = _BV(PIOTXD0) | _BV(PIORXD0);

    /* Reset tranmitter and receiver. */
    usart0_reg[AT91_US_CR] = _BV(AT91_US_RSTRX) | _BV(AT91_US_RSTTX) |
                             _BV(AT91_US_RXDIS) | _BV(AT91_US_TXDIS);

    /* Disable all interrupts. */
    usart0_reg[AT91_US_IDR] = 0xFFFFFFFF;

    /* Clear transmit and receiver counter. */
    usart0_reg[AT91_US_RCR] = 0;
    usart0_reg[AT91_US_TCR] = 0;

    /* Set baud rate, MCKI / (16 * rate) */
    usart0_reg[AT91_US_BRG] = 40; /*AT91_US_BAUD(115200); */

    /* 8N1, no parity. */
    usart0_reg[AT91_US_MR] = AT91_US_MR_LENGTH_8 | AT91_US_MR_PARITY_NONE;

    /* Enable receiver and transmitter. */
    usart0_reg[AT91_US_CR] = _BV(AT91_US_RXEN) | _BV(AT91_US_TXEN);
}

void TraceChar(char ch)
{
    usart0_reg[AT91_US_THR] = ch;
    while((usart0_reg[AT91_US_CSR] & _BV(AT91_TXRDY)) == 0);

    if(ch == '\n') {
        TraceChar('\r');
    }
}

void TraceString(char *buf)
{
    while(*buf) {
        TraceChar(*buf++);
    }
}

void TraceShort(unsigned short val)
{
    register unsigned int i;

    for (i = 4; i-- > 0;) {
        hexbuf[i] = hexdigit[val & 0x0F];
        val >>= 4;
    }
    hexbuf[4] = 0;
    TraceString(hexbuf);
}

void TraceLong(unsigned long val)
{
    register unsigned int i;

    for (i = 8; i-- > 0;) {
        hexbuf[i] = hexdigit[val & 0x0F];
        val >>= 4;
    }
    hexbuf[8] = 0;
    TraceString(hexbuf);
}

