/*!
 * Copyright (C) 2005 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://jtagomat.sourceforge.net/
 */

/*!
 * $Log: comm.c,v $
 * Revision 1.1  2006/01/09 09:16:52  haraldkipp
 * First check-in.
 *
 */

#include <avr/io.h>

#include "comm.h"

#ifdef __AVR_ATmega128__
#define UBRRL   UBRR0L
#define UDR     UDR0
#define UCSRA   UCSR0A
#define UCSRB   UCSR0B
#endif

#define STX     0x02
#define ETX     0x03
#define ACK     0x06
#define DLE     0x10
#define NAK     0x15

/*!
 * \brief Enable UART communication.
 */
void InitUart(void)
{
#ifdef __AVR_ATmega128__
    UBRRL = 7;
#else
    UBRRL = 3;
#endif
    UCSRB = (1 << RXEN) | (1 << TXEN);
}

static void SendChar(unsigned char ch)
{
    while((UCSRA & _BV(UDRE)) == 0);
    UDR = ch;
}

/*!
 * \brief Receive an STX/ETX/CKS telegram.
 */
int RecvTelegram(unsigned char *buf, int siz)
{
    int rc = 0;
    unsigned short tmo = 0;
    unsigned char ch;
    unsigned char cs = 0;
    unsigned char st = 0;

    for (;;) {
        while((UCSRA & _BV(RXC)) == 0) {
            if (tmo && --tmo == 0) {
                return -1;
            }
        }
        ch = UDR;
        cs ^= ch;
        tmo = 0xFFFF;
        if (st == 1) {
            if (ch == STX) {
                st = 0;
            }
            else if (ch == DLE) {
                st = 2;
                cs ^= ch;
                continue;
            }
            else if (ch == ETX) {
                st = 3;
                continue;
            }
        }
        if (st == 0) {
            if (ch == STX) {
                cs = 0;
                st++;
            }
            else if (ch == 22) {
                char *vm = "Turtelizer " VERSION_STRING "\r\n";
                while (*vm) {
                    SendChar(*vm++);
                }
            }
            continue;
        }
        if (st == 3) {
            if (cs) {
                rc = -1;
                SendChar(NAK);
            }
            else {
                SendChar(ACK);
            }
            break;
        }
        if (rc >= siz) {
            rc = -1;
            break;
        }
        buf[rc] = ch;
        rc++;
        st = 1;
    }
    return rc;
}

/*!
 * \brief Send an STX/ETX/CKS telegram.
 */
int SendTelegram(const unsigned char *buf, int len)
{
    int rc = 0;
    unsigned short tmo;
    unsigned char ch;
    unsigned char cs = 0;

    SendChar(STX);
    for (rc = 0; rc < len; rc++) {
        ch = buf[rc];
        cs ^= ch;
        if (ch == STX || ch == ETX || ch == DLE) {
            SendChar(DLE);
        }
        SendChar(ch);
    }
    SendChar(ETX);
    cs ^= ETX;
    SendChar(cs);

    rc = -1;
    tmo = 0xFFFF;
    while((UCSRA & _BV(RXC)) == 0) {
        if (tmo && --tmo == 0) {
            break;
        }
    }
    if (tmo) {
        ch = UDR;
        if (ch == ACK) {
            rc = 0;
        }
    }
    return rc;
}

